import time
import logging
import smtplib

from email.mime.multipart import MIMEMultipart
from email.mime.image import MIMEImage

import cv2


logging.basicConfig(
    level=logging.DEBUG, format="%(asctime)s %(levelname)-8s %(message)s"
)

# Set up camera capture
camera = cv2.VideoCapture(1)

# Define motion detection threshold
MOTION_THRESHOLD = 5000
MOTION_TIMEOUT = 10  # seconds

# Define email parameters
SMTP_SERVER = "smtp.gmail.com"
SMTP_PORT = 587
SMTP_USERNAME = "your-email@gmail.com"
SMTP_PASSWORD = "your-email-password"
FROM_EMAIL = "your-email@gmail.com"
TO_EMAIL = "recipient-email@example.com"
SUBJECT = f"No motion detected for {MOTION_TIMEOUT} seconds"


# Define function to send email notification
def send_email(image):
    # Create message
    message = MIMEMultipart()
    message["From"] = FROM_EMAIL
    message["To"] = TO_EMAIL
    message["Subject"] = SUBJECT

    # Attach image to message
    encoded_image = cv2.imencode(".jpg", image)[1].tobytes()
    image_attachment = MIMEImage(encoded_image)
    message.attach(image_attachment)

    # Send message
    with smtplib.SMTP(SMTP_SERVER, SMTP_PORT) as server:
        server.starttls()
        server.login(SMTP_USERNAME, SMTP_PASSWORD)
        server.sendmail(FROM_EMAIL, TO_EMAIL, message.as_string())


# Define variables for motion detection loop
last_frame = None
last_motion_time = time.time()

# Motion detection loop
while camera.isOpened():
    logging.debug("Starting next round")
    time.sleep(1)
    # Capture frame from camera
    ret, frame = camera.read()

    # Convert frame to grayscale and blur it
    gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    gray = cv2.GaussianBlur(gray, (21, 21), 0)

    # If this is the first frame, set it as the last frame and continue
    if last_frame is None:
        last_frame = gray
        continue

    # Calculate absolute difference between the current frame and the last frame
    frame_diff = cv2.absdiff(last_frame, gray)

    # Threshold the difference image to obtain binary image
    thresh = cv2.threshold(frame_diff, 25, 255, cv2.THRESH_BINARY)[1]

    # Dilate the thresholded image to fill in holes
    thresh = cv2.dilate(thresh, None, iterations=2)

    # Calculate contours in the thresholded image
    contours, _ = cv2.findContours(
        thresh.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE
    )

    # Loop over the contours
    motion_detected = False
    for contour in contours:
        # If contour area is smaller than threshold, ignore it
        if cv2.contourArea(contour) < MOTION_THRESHOLD:
            logging.debug("Change area to small, ignoring")
            continue

        # Motion detected
        logging.debug("Motion detected, starting next round")
        motion_detected = True
        break

    # Update last motion time if motion is detected
    if motion_detected:
        last_motion_time = time.time()

    # Send email notification if no motion is detected for more than 60 seconds
    if not motion_detected and time.time() - last_motion_time > MOTION_TIMEOUT:
        logging.debug("Sending notification")
        send_email(frame)
        last_motion_time = time.time()

    # Set current frame as last frame for next iteration
    last_frame = gray

    # Display camera feed
    cv2.imshow("Camera", frame)

    # Exit loop if 'q' is pressed
    if cv2.waitKey(1) & 0xFF == ord("q"):
        break

# Release camera and close windows
